use crate::{
	result::{errorlog, into_response_helper, ResourceError},
	OpenapiSchema, ResourceResult, Response
};
use futures_util::future::{FutureExt, TryFutureExt};
use gotham::hyper::StatusCode;
use mime::Mime;
use openapiv3::{SchemaKind, StringFormat, StringType, Type, VariantOrUnknownOrEmpty};
use std::{
	future::Future,
	path::PathBuf,
	pin::Pin
};
use thiserror::Error;

pub struct File
{
	path : PathBuf,
	content_type : Option<Mime>
}

impl ResourceResult for File
{
	type Err = tokio::io::Error;
	
	fn into_response(self) -> Pin<Box<dyn Future<Output = Result<Response, Self::Err>> + Send>>
	{
		async move {
			let data = tokio::fs::read(self.path).await?;
			let res = Response::new(StatusCode::OK, data, self.content_type);
			Ok(res)
		}.boxed()
	}
	
	#[cfg(feature = "openapi")]
	fn schema() -> OpenapiSchema
	{
		OpenapiSchema::new(SchemaKind::Type(Type::String(StringType {
			format: VariantOrUnknownOrEmpty::Item(StringFormat::Binary),
			..Default::default()
		})))
	}
}

#[derive(Debug, Error)]
pub enum IoOrJsonError
{
	#[error("{0}")]
	IoError(#[from] tokio::io::Error),
	#[error("{0}")]
	JsonError(#[from] serde_json::Error)
}

impl<E : std::error::Error + Send + 'static> ResourceResult for Result<File, E>
{
	type Err = IoOrJsonError;
	
	fn into_response(self) -> Pin<Box<dyn Future<Output = Result<Response, IoOrJsonError>> + Send>>
	{
		match self {
			Ok(f) => f.into_response().map_err(|e| e.into()).boxed(),
			Err(e) => into_response_helper(|| {
				errorlog(&e);
				let err : ResourceError = e.into();
				Ok(Response::json(StatusCode::INTERNAL_SERVER_ERROR, serde_json::to_string(&err)?))
			})
		}
	}
	
	#[cfg(feature = "openapi")]
	fn schema() -> OpenapiSchema
	{
		OpenapiSchema::new(SchemaKind::Type(Type::String(StringType {
			format: VariantOrUnknownOrEmpty::Item(StringFormat::Binary),
			..Default::default()
		})))
	}
}
